// ############################
// # FIRMWARE 0.1a MP3-Wecker #
// #                          #
// # by hannes eilers         #
// ############################

#define F_CPU 8000000UL				// CPU-Geschwindigkeit

// INCLUDES
#include <avr/io.h>
#include <stdint.h>
#include <string.h>
#include <avr/iom8.h>
#include <util/delay.h>
#include <inttypes.h>
#include <stdlib.h>
#include <avr/interrupt.h> 


// DEFINITIONEN
#define CLOCK_ADDR 0xA0				// Adresse Uhrenbaustein
#define IO_ADDR 0x40				// Adresse IO-Expander

#define INT_ALARM	INT0_vect 		// Interrupt-Vektor fr Alarm-Interrupt
#define INT_KEYBRD	INT1_vect 		// Interrupt-Vektor fr Tastatur-Interrupt
#define INT_TMR0	TIMER0_OVF_vect	// Interrupt-Vektor fr Timer0 Overflow
#define INT_TMR1	TIMER1_OVF_vect	// Interrupt-Vektor fr Timer1 Overflow
#define INT_TMR2	TIMER2_OVF_vect	// Interrupt-Vektor fr Timer2 Overflow

// LED Ports
#define LED_R			PD7
#define LED_G			PD6
#define LED_B			PD5
#define LED_BRIGHT		PC2

// PWM-Einstellungen
// PWM_COUNT_X = F_CPU / (Prescaler * 256^2 * F_PWM)
#define PWM_COUNT_LED		1			// PWM Durchlufe LED-PWM fr 50Hz PWM bei 8MHz CPU-Takt
#define PWM_COUNT_BRIGHT	1			// PWM Durchlufe Helligkeit fr 100Hz
#define PWM_COUNT_LCD		4			// PWM Durchlufe LCD-Anzeige in Sekunden(beginnend ab 0)

#define ASCII_L 9					// Stringlnge (1 Byte) in ASCII

// DEFENITION ALARM WEEKDAYS: AL_WKDAYS
#define AL_WK_SO	0x01
#define AL_WK_MO	0x02
#define AL_WK_DI	0x04
#define AL_WK_MI	0x08
#define AL_WK_DO	0x10
#define AL_WK_FR	0x20
#define AL_WK_SA	0x40

// GLOBALE VARIABELN
uint8_t flash_time = 0;					// Zeitanzeige mit Flash?
uint8_t edit_time = 0;					// Aufruf von get_time() bei Zeitanzeige bergehen

// Farvariabeln
uint8_t color_switch_mode = 0;			// Geschwindigkeit fr LED-Farbwechsel(mode: 0, 2, 4, 8, 16);
uint8_t alarm_color_switch_mode = 0;	// Geschwindigkeit fr Alarm-Lichtwechsel


// STRUKTUREN

// Farbstruktur
struct color_struc{
	uint8_t R;
	uint8_t G;
	uint8_t B;
	uint8_t brightness;
};

// Zeit-Struktur
struct time_struc{
	uint8_t thours;
	uint8_t nhours;

	uint8_t tmin;
	uint8_t nmin;

	uint8_t tsec;
	uint8_t nsec;

	uint8_t tday;
	uint8_t nday;

	uint8_t tmon;
	uint8_t nmon;

	uint8_t wkday;

	uint8_t year;
};


// Eigene Struktutvariabeln
struct time_struc timestamp;
struct time_struc new_timestamp;
struct time_struc edit_timestamp;

struct color_struc led_color;
struct color_struc alarm_color;


// INCLUDES
#include "lcd_routines.h"			// LCD-Steuerung
#include "i2cmaster.h"				// I2C-Steuerung
#include "functions.h"				// diverse Funktionen
#include "pcf8583.h"				// Funktionen fr Uhrenbaustein
#include "interrupts.h"				// Interrupt Funktionen
#include "menue.h"					// Menustruktur & -funktionen


// FUNKTIONEN

// -- MAIN
int main(){

	led_init();						// LEDs initialisieren
	lcd_init();						// LCD-Display initialisieren
	i2c_init();						// I2C initialisieren
	keybrd_init();					// Tastatur & MP3-Player initialisieren
	init_menue();					// Men erstellen

	set_led_color(0, 0, 0, 0, &led_color);		// LEDs ausschalten
	
	set_date_time();

	keybrd = twi_read( IO_ADDR, 0x00 );
	keybrd = 0x00;

	// Alarm setzen
	new_timestamp.nsec = 9;

	//set_alarm(1);

	// TEST
	// TEST ENDE
	

	act_int();						// Interrupts einschalten

	while(1){

		// ----------------------------------------------------------------------
		// INTERRUPTROUTINEN AUSFHREN
			if( _TMR0 ) _tmr0();
			if( _TMR1 ) _tmr1();
			if( _TMR2 ) _tmr2();
		// ----------------------------------------------------------------------


		// Zeit- & Datumsanzeige
		if( step_lcd < 15 && flash_time != 1 ){
				
				// Datum anzeigen
				set_cursor(3, 1);
				show_wkday();
				show_date();
				

		}
		else if( step_lcd >= 15 || flash_time == 1 ){

			set_cursor(4, 1);
			show_time();

		}

		if( step_lcd == 0 || (cnt_lcd == 1 && flash_time == 1) || step_lcd == 15 ){
			lcd_clear();
		}


		// Lichtwechsel
		if( step_lcd % 5 == 0 && step_lcd != 0 ){
				
				// Auf automatischen Lichtwechsel prfen
				if( color_switch_mode != 0 && flash_time != 1) led_automatic_change_color(color_switch_mode);
				if( alarm_color_switch_mode != 0 && flash_time == 1 ) led_automatic_change_color(alarm_color_switch_mode);

		}

		// ggf.Men aufrufen
		if( keybrd ){

			keybrd = 0x00;

			menue();

			keybrd = 0x00;
			edit_time = 0;

			lcd_clear();

		}
		
		_delay_ms(100);

	}


}



