// ######################
// # EXTERNE FUNKTIONEN #
// ######################


// Umwandlung eines Integers zu einem String und Ausgabe via lcd_string()
void lcd_itoa(uint8_t x){

	char BUFF[10];
	itoa( x, BUFF, 10 );
	lcd_string( BUFF );

	return;

}


// Automatischer LED Farbwechsel
void led_automatic_change_color(uint8_t mode){

	uint8_t i = mode;
	static uint8_t ovf_r = 0, ovf_g = 0, ovf_b = 0;

	led_color.brightness = 255;

	if( ovf_r == 0 && ovf_g == 0 ) led_color.R = led_color.R + i;
	if( ovf_r == 1 ) led_color.R = led_color.R - i;

	if( ovf_r == 1 && ovf_b == 0 ) led_color.G = led_color.G + i;
	if( ovf_g == 1 ) led_color.G = led_color.G - i;

	if( ovf_g == 1 && ovf_r == 0 ) led_color.B = led_color.B + i;
	if( ovf_b == 1 ) led_color.B = led_color.B - i;

	if( led_color.R >= 255 ) ovf_r = 1;
	if( led_color.G >= 255 ) ovf_g = 1;
	if( led_color.B >= 255 ) ovf_b = 1;

	if( led_color.R <= 0 ) ovf_r = 0;
	if( led_color.G <= 0 ) ovf_g = 0;
	if( led_color.B <= 0 ) ovf_b = 0;

}


void set_led_color(uint8_t r, uint8_t g, uint8_t b, uint8_t bright, struct color_struc *element){

	element -> R = r;
	element -> G = g;
	element -> B = b;
	element -> brightness = bright;

}

// Initialisiert LEDs
void led_init(){

	DDRD = (1 << LED_R) | (1 << LED_G) | (1 << LED_B);	// PD5-7 als Ausgang
	PORTD = (1 << LED_R) | (1 << LED_G) | (1 << LED_B);	// PD5-7 auf High (LEDs aus)

	DDRC = (1 << LED_BRIGHT);							// PC2 als Ausgang
	PORTC = (1 << LED_BRIGHT);							// PC2 auf High (Helligkeit 0%)

	// LED-Farbe setzen
	set_led_color(0, 0, 0, 0, &led_color);

}



// Interrupts fr Alarm und Tastatur einschalten
void act_int(){

	// Interrupt INT0 & INT1
	DDRD &= ~(1 << PD2) | (1 << PD3);		// Interrupt Pins als Input

	GIMSK = 0xC0;							// INT0 & INT1 Interrupt aktivieren
	MCUCR = 0x0A;							// uint8_t 0 fallende, INT1 steigende Flanke

	// interrupt TIMER0 und TIMER2
	TCCR0 = (1 << CS00);									// Timer0 an mit Prescaler 1
	TCCR2 = (1 << CS20);									// Timer2 an mit Prescaler 1
	TCCR1A = 0x00;											// Timer1 als normaler Timer
	TCCR1B = (1 << CS11);									// Timer1 an mit Prescaler 8
	TIMSK = (1 << TOIE0) | (1 << TOIE2) | (1 << TOIE1);		// Timer-Overflow Interrupt einschalten

	sei();													// Interrupts generell einschalten

}



// Ein Byte ber I2C lesen
uint8_t twi_read(uint8_t client_addr, uint8_t addr){

	uint8_t ret;

	// -- Tag
	i2c_start_wait(client_addr + I2C_WRITE);		// set device address and write mode

    i2c_write(addr);                        		// set ram-address
    i2c_rep_start(client_addr + I2C_READ);       	// set device address and read mode

    ret = i2c_readNak();                    		// read one byte
    i2c_stop();

	return ret;

}

// Ein Byte ber I2C schreiben
void twi_write(uint8_t client_addr, uint8_t addr, uint8_t data){

	i2c_start_wait(client_addr + I2C_WRITE);     // set device address and write mode
	_delay_us(15);
    i2c_write(addr);                        	// set ram-address
	_delay_us(15);
    i2c_write(data);                        	// write data to register
	_delay_us(15);
    i2c_stop();                             	// set stop conditon = release bus
	_delay_us(15);

}


// Initialisierung der Tastatur & des MP3-Player
void keybrd_init(){

	// Standartwerte fr Tastaturinputs auf 0 setzen
	twi_write( IO_ADDR, 0x02,0x00 );

	// MP3-Player Port als Ausgang & Standartwerte auf 0
	twi_write( IO_ADDR, 0x07, 0x00 );
	twi_write( IO_ADDR, 0x03, 0x00 );

}


void get_time(){

	// Auslesen der Daten aus dem Uhrenbaustein
	uint8_t ret;

	// -- Stunden
	ret = twi_read(CLOCK_ADDR, 0x04);

		// Daten umwandeln
		timestamp.thours = ret >> 4;					// oberes Nibble holen
		timestamp.thours = timestamp.thours & 0x03;		// letzten 2 Bit maskieren
		timestamp.nhours = ret & 0x0F;					// unteres Nibble holen



	// -- Minuten
	ret = twi_read(CLOCK_ADDR, 0x03);
	
		// Daten umwandeln
		timestamp.tmin = ret >> 4;					// oberes Nibble holen
		timestamp.tmin = timestamp.tmin & 0x07;	// letzten 3 Bit maskieren
		timestamp.nmin = ret & 0x0F;				// unteres Nibble holen



	// -- Sekunden
	ret = twi_read(CLOCK_ADDR, 0x02);
	
		// Daten umwandeln
		timestamp.tsec = ret >> 4;						// oberes Nibble holen
		timestamp.tsec = timestamp.tsec & 0x07;			// letzten 3 Bit maskieren
		timestamp.nsec = ret & 0x0F;					// unteres Nibble holen

	// -- Tag
	ret = twi_read(CLOCK_ADDR, 0x05);

		// Daten umwandeln
		timestamp.tday = ret >> 4;						// oberes Nibble holen
		timestamp.tday = timestamp.tday & 0x03;			// letzten 2 Bit maskieren
		timestamp.nday = ret & 0x0F;					// unteres Nibble holen

	
	// -- Monat
	ret = twi_read(CLOCK_ADDR, 0x06);

		// Daten umwandeln
		timestamp.tmon = ret >> 4;						// oberes Nibble holen
		timestamp.tmon = timestamp.tmon & 0x01;			// letzten 2 Bit maskieren
		timestamp.nmon = ret & 0x0F;					// unteres Nibble holen

		timestamp.wkday = ret >> 5;						// oberen 3 Bit holen 0x06
		timestamp.wkday = timestamp.wkday & 0x07;		// letze 3 Bit maskieren

	
}


// -- Ausgabe des Wochentags
void show_wkday(){

	get_time();						// aktuelle Zeit auslesen
	edit_timestamp = timestamp;
	if( edit_time ) edit_timestamp = new_timestamp;

	// Wochentag anzeigen
	switch (edit_timestamp.wkday){

		case 0:
			lcd_string("So.");
			break;
		
		case 1:
			lcd_string("Mo.");
			break;
			
			
		case 2:
			lcd_string("Di.");
			break;
			
		case 3:
			lcd_string("Mi.");
			break;
			
		case 4:
			lcd_string("Do.");;
			break;
			
		case 5:
			lcd_string("Fr.");
			break;	
			
		case 6:
			lcd_string("Sa.");
			break;	

	}

}


// -- Ausgabe des Datum
void show_date(){

	// Auslesen der Daten aus dem Uhrenbaustein
	get_time();						// Aktuelle Zeit auslesen
	edit_timestamp = timestamp;

	if( edit_time ) edit_timestamp = new_timestamp; 

	lcd_itoa(edit_timestamp.tday);		// Zehnert-Tag
	lcd_itoa(edit_timestamp.nday);		// Einer-Tag

	lcd_string( "." );									// Punkte

	lcd_itoa(edit_timestamp.tmon);		// Zehner-Monat
	lcd_itoa(edit_timestamp.nmon);		// Einer-Monat

	lcd_string(".");									//Punkt


}


// -- Ausgeben der Uhrzeit
void show_time(){

	// Auslesen der Daten aus Uhrenbaustein
	if( !flash_time ) get_time();		// Aktuelle Zeit auslesen	
	edit_timestamp = timestamp;

	if( edit_time ) edit_timestamp = new_timestamp;

	
	lcd_itoa(edit_timestamp.thours);	// Zehner-Stunde
	lcd_itoa(edit_timestamp.nhours);	// Einer-Stunde

	lcd_string(":");									// Doppelpunkt

	lcd_itoa(edit_timestamp.tmin);		// Zehner-Minutes
	lcd_itoa(edit_timestamp.nmin);		// Einer-Minutes	

	lcd_string(":");									// Doppelpunkt

	lcd_itoa(edit_timestamp.tsec);		// Zehner-Sekunde
	lcd_itoa(edit_timestamp.nsec);		// Einer-Sekunde

}
