// ##########################
// # Uhrbaustein Funktionen #
// ##########################

 
void start_after_reset(){

	// Einstellungen nach System reset setzen
	// 	- counting
	//	- count
	//	- clock mode
	//	- mask
	//	- alarm enable
	//	- no flags set

	twi_write(CLOCK_ADDR, 0x00, 0x0C);			// write 0x0C to control register (0x00)

}


// ALARM SETZEN
// type:
// no alarm: 0
// daily: 1
// weekdays: 2 (needs set of AL_WKDAYS as new_timestamp.wkday)
void set_alarm(uint8_t type){

	if( type == 1 )			twi_write(CLOCK_ADDR, 0x08, 0x90 );		// Alarm aktivieren
	else if( type == 2 )  	twi_write(CLOCK_ADDR, 0x08, 0xA0 );		// Alarm aktivieren
	else 					twi_write(CLOCK_ADDR, 0x08, 0x00 );		// Alarm aktivieren
	
	// HSekunden
	twi_write(CLOCK_ADDR, 0x09, 0x00);

	// Sekunden
	twi_write(CLOCK_ADDR, 0x0A, (new_timestamp.tsec << 4) + new_timestamp.nsec);


	// Minutes
	twi_write(CLOCK_ADDR, 0x0B, (new_timestamp.tmin << 4) + new_timestamp.nmin);
	
	// Stunden
	twi_write(CLOCK_ADDR, 0x0C, (new_timestamp.thours << 4) + new_timestamp.nhours);

	// Tag & Jahr
	twi_write(CLOCK_ADDR, 0x0D, (new_timestamp.year << 7) + (new_timestamp.tday << 4) + new_timestamp.nday);


	// Wochentag & Monat
	twi_write(CLOCK_ADDR, 0x0E, (new_timestamp.wkday << 7) + (new_timestamp.tmon << 4) + new_timestamp.nmon);


}


// DATUM SETZEN
void set_date_time(){

	// Datum speichern

	// -- Statusregister auslesen
	uint8_t ret;

	ret = twi_read(CLOCK_ADDR, 0x00);

	// -- Zhler stoppen
	twi_write(CLOCK_ADDR, 0x00, ret + 0x80);


	// -- Datum setzen

	// HSekunden
	twi_write(CLOCK_ADDR, 0x01, 0x00);

	// Sekunden
	twi_write(CLOCK_ADDR, 0x02, (new_timestamp.tsec << 4) + new_timestamp.nsec);


	// Minutes
	twi_write(CLOCK_ADDR, 0x03, (new_timestamp.tmin << 4) + new_timestamp.nmin);
	
	// Stunden
	twi_write(CLOCK_ADDR, 0x04, (new_timestamp.thours << 4) + new_timestamp.nhours);

	// Tag & Jahr
	twi_write(CLOCK_ADDR, 0x05, (new_timestamp.year << 7) + (new_timestamp.tday << 4) + new_timestamp.nday);

	// Wochentag & Monat
	twi_write(CLOCK_ADDR, 0x06, (new_timestamp.wkday << 7) + (new_timestamp.tmon << 4) + new_timestamp.nmon);
	

	// Zhler wieder starten
	start_after_reset();

	get_time();

}


// Ermitteln der fr den Uhrenbaustein gltigen Jahreszahl durch eine vierstellige Jahreszahl
uint8_t get_year(uint8_t year){

	// Ist Jahr ein Schaltjahr?
	if( ( (year % 4) && !(year % 100) ) || (year % 400) ) return 0;			// Schaltjahr
	
	// kein Schaltjahr
	// --> Jahr vor dem gegeb. Jahr berprfen
	
	if( ( ((year - 1) % 4) && !((year - 1) % 100) ) || ((year - 1) % 400) ) return 1;			// Jahr - 1 = Schaltjahr
	if( ( ((year - 2) % 4) && !((year - 2) % 100) ) || ((year - 2) % 400) ) return 2;			// Jahr - 2 = Schaltjahr
	if( ( ((year - 3) % 4) && !((year - 3) % 100) ) || ((year - 3) % 400) ) return 3;			// Jahr - 3 = Schaltjahr

	return 0;

}
